/*************** sugeno_eval.c *********/
/*** c functions for evaluating sugeno fuzzy inference systems ****/
/*  Jason Seawall  */
/*  Applied Research Labs */
/*  9-15-01 */
//
//
/* first define the following variables using your matlab fis values
/* see example code fis.c for an example

	int InputsOutputs[]
    int NumInputMFs[]
	int NumOutputMFs
	int NumRules

    *** matlabs defualt values for the following values are used
		AndMethod,OrMethod,ImpMethod,AggMethod,DefuzzMethod

    float InRange[]
	float OutRange[]

    int InMFTypes[] ********(1=trimf,2=bellmf,3=gaussmf)

    *** matlabs defualt values for the following values are used
		OutMFTypes

    float InMFParams[]
	float OutMFParams[]


*/		
// with the fis variables defined, call the fuzzy_eval() function


#include <float.h>
#include <math.h>

 

///////////////
// membership functions (1=trimf,2=bellmf,3=gaussmf)
// % add other mebership functions here, and modify get_fuzzy
//   as needed
///////////////

float trimf(float x,float *mf){
    float y;
    
    if(x>=mf[0] && x<=mf[1]){
       y= (x-mf[0])/(mf[1]-mf[0]);
    }
    else if(x>mf[1] && x<=mf[2]){
       y= (mf[2]-x)/(mf[2]-mf[1]);
    }
    else y=0;
    return(y);
} // triangle membership function

float bellmf(float x,float *mf){
    float y;
    float a,b,c,tmp;
    a = mf[0];
    b = mf[1];
    c = mf[2];

   tmp = ((x - c)/a)*((x - c)/a);
   if (tmp == 0 && b == 0)
      y = 0.5;
   else if (tmp == 0 && b < 0)
      y = 0;
   else {
      tmp=pow(tmp,b);
      y = 1/(1 + tmp);
   }
   return(y);
} // bell membership function

float gaussmf(float x,float *mf){
   float y;
   float sigma,c;
   sigma = mf[1];
   c = mf[2];
   y = exp( -pow((x - c),2) / (pow(2*sigma,2)) );
   return(y);
}


float get_fuz(int type,float *mf,float x){
   float y;
   if(type==1)y=trimf(x,mf);
   else if(type==2)y=bellmf(x,mf);
   else if(type==2)y=gaussmf(x,mf);
   return(y);
} // get fuzzy input


///////////////
// fis processing
///////////////




int fuzzy_eval(float *crisp_in,float *crisp_out){
    int i,j,k;
	int base,count,Nmfout,ant_value,dbug;
	float mf[4];
	float fuz_in[10],mfout_single[11]; // this may need to be increased for larger systems
	float antecedent;
	float atotal,ftotal;
    float fuz_out;
 
    // check that input is within range
    for(i=0;i<InputsOutputs[0];i++){ // loop through crisp inputs
		if( crisp_in[i]<InRange[i*2] | crisp_in[i]>InRange[i*2+1]){
//			print('input is not within range');
			return(1); // not successful
		}
	
	}
    count=0;
    for(i=0;i<InputsOutputs[0];i++){ // loop through crisp inputs
		for(j=0;j<NumInputMFs[i];j++){ // loop through MFs for each input rule 
			for(k=0;k<4;k++){ // loop through four MF values
             mf[k]= InMFParams[(count)*4+k]; // set of MF values for single rule
			} // get mf parameters
          fuz_in[count]=get_fuz(InMFTypes[count],mf,crisp_in[i]);//get fuzzy inputs
	      count=count+1;

       } // get fuzzy input for each mf
    } //loop through crisp inputs
   atotal=0;
   ftotal=0;
   
   if(InputsOutputs[0]<4)Nmfout=4;
   else Nmfout=InputsOutputs[0]+1;
  
    for(i=0;i<NumOutputMFs;i++){ // loop through output rules
	   	for(j=0;j<(InputsOutputs[0]+1);j++){ // output MF for each input plus one
           mfout_single[j]=OutMFParams[i*(Nmfout) + j];

        }


		base=0;
		antecedent=1;
		fuz_out=0;
	    for(j=0;j<InputsOutputs[0];j++){ // get set of fuzzy inputs for currnet output rule
		   ant_value=base+Rule_Antecedent[i*InputsOutputs[0]+j]-1;
           antecedent=antecedent*fuz_in[ant_value];
		   base=base+NumInputMFs[j];
		   fuz_out=fuz_out+crisp_in[j]*mfout_single[j]; // get fuzzy output

        }
		fuz_out=fuz_out+mfout_single[(InputsOutputs[0])]; // add constant output MF value

	    atotal=atotal+antecedent;
        ftotal=ftotal+fuz_out*antecedent;
                
    }

    *crisp_out=ftotal/atotal;
	
  
    return 0; // successful

    
}


